@@ -55,14 +55,12 @@ void algo_run()
   count_comparisons_8 = 0;
   count_comparisons_16 = 0;
 
-  unsigned long searches = 0;
-
 #ifdef VERBOSE
+  unsigned long searches = 0;
   unsigned long estimates = 0;
 #endif
 
   unsigned long largestswarm = 0;
-  unsigned long swarmsize = 0;
 
   unsigned long maxgenerations = 0;
 
@@ -113,7 +111,7 @@ void algo_run()
   
   unsigned long bits;
 
-  if ((unsigned long)resolution <= diff_saturation)
+  if ((unsigned long)opt_differences <= diff_saturation)
     bits = 8;
   else
     bits = 16;
@@ -131,6 +129,7 @@ void algo_run()
       
       swarmid++;
 
+      unsigned long swarmsize = 0;
       unsigned long amplicons_copies = 0;
       unsigned long singletons = 0;
       unsigned long hitcount = 0;
@@ -184,7 +183,7 @@ void algo_run()
           unsigned poolampliconid = qgramamps[i];
           long diff = qgramdiffs[i];
           amps[swarmed+i].diffestimate = diff;
-          if (diff <= resolution)
+          if (diff <= opt_differences)
             {
               targetindices[targetcount] = swarmed+i;
               targetampliconids[targetcount] = poolampliconid;
@@ -196,7 +195,9 @@ void algo_run()
         {
           search_do(seedampliconid, targetcount, targetampliconids, 
                     scores, diffs, alignlengths, bits);
+#ifdef VERBOSE
           searches++;
+#endif
 
           if (bits == 8)
             count_comparisons_8 += targetcount;
@@ -218,7 +219,7 @@ void algo_run()
 
               unsigned diff = diffs[t];
 
-              if (diff <= (unsigned long) resolution)
+              if (diff <= (unsigned long) opt_differences)
                 {
                   unsigned i = targetindices[t];
 
@@ -294,7 +295,7 @@ void algo_run()
               for(unsigned long i=swarmed; i<amplicons; i++)
                 {
                   unsigned long targetampliconid = amps[i].ampliconid;
-                  if ((amps[i].diffestimate <= subseedradius + resolution) &&
+                  if ((amps[i].diffestimate <= subseedradius + opt_differences) &&
                       ((opt_no_otu_breaking) || 
                        (db_getabundance(targetampliconid)
                         <= subseedabundance)))
@@ -313,7 +314,7 @@ void algo_run()
 #endif
 
               for(unsigned long i=0; i < listlen; i++)
-                if ((long)qgramdiffs[i] <= resolution)
+                if ((long)qgramdiffs[i] <= opt_differences)
                   {
                     targetindices[targetcount] = qgramindices[i];
                     targetampliconids[targetcount] = qgramamps[i];
@@ -324,7 +325,9 @@ void algo_run()
                 {
                   search_do(subseedampliconid, targetcount, targetampliconids, 
                             scores, diffs, alignlengths, bits);
+#ifdef VERBOSE
                   searches++;
+#endif
 
                   if (bits == 8)
                     count_comparisons_8 += targetcount;
@@ -335,7 +338,7 @@ void algo_run()
                     {
                       unsigned diff = diffs[t];
               
-                      if (diff <= (unsigned long) resolution)
+                      if (diff <= (unsigned long) opt_differences)
                         {
                           unsigned i = targetindices[t];
                 
@@ -486,12 +489,12 @@ void algo_run()
       char sep_amplicons;
       char sep_swarms;
 
-      if (mothur)
+      if (opt_mothur)
         {
           /* mothur list file output */
           sep_amplicons = ',';
           sep_swarms = '\t';
-          fprintf(outfile, "swarm_%ld\t%lu\t", resolution, swarmid);
+          fprintf(outfile, "swarm_%ld\t%lu\t", opt_differences, swarmid);
         }
       else
         {
