@@ -42,7 +42,7 @@ static struct ampinfo_s
   int parent;
   int generation; 
   int next;       /* amp id of next amplicon in swarm */
-  int graft_cand; /* amp id of potential grafting parent (fastitdious) */
+  int graft_cand; /* amp id of potential grafting parent (fastidious) */
 } * ampinfo = 0;
 
 /* Information about each swarm (OTU) */
@@ -375,7 +375,7 @@ void * worker(void * vp)
   while (tip->work >= 0)
     {
       /* wait for work available */
-      if (tip->work == 0)
+      while (tip->work == 0)
         pthread_cond_wait(&tip->workcond, &tip->workmutex);
       if (tip->work > 0)
         {
@@ -397,10 +397,10 @@ void threads_init()
   /* allocate memory for thread info, incl the variant sequences */
   unsigned long longestamplicon = db_getlongestsequence();
   ti = (struct thread_info_s *) 
-    xmalloc(threads * sizeof(struct thread_info_s));
+    xmalloc(opt_threads * sizeof(struct thread_info_s));
   
   /* init and create worker threads */
-  for(unsigned long t=0; t<threads; t++)
+  for(long t=0; t<opt_threads; t++)
     {
       struct thread_info_s * tip = ti + t;
       tip->varseq = (unsigned char*) xmalloc(longestamplicon+1);
@@ -417,7 +417,7 @@ void threads_init()
 void threads_done()
 {
   /* finish and clean up worker threads */
-  for(unsigned long t=0; t<threads; t++)
+  for(long t=0; t<opt_threads; t++)
     {
       struct thread_info_s * tip = ti + t;
       
@@ -442,37 +442,18 @@ void threads_done()
   pthread_attr_destroy(&attr);
 }
 
-void swarm_breaker_info(int amp)
-{
-  /* output info for swarm_breaker script */
-  if (opt_internal_structure)
-    {
-      long seed = ampinfo[amp].parent;
-      fprint_id_noabundance(internal_structure_file, seed);
-      fprintf(internal_structure_file, "\t");
-      fprint_id_noabundance(internal_structure_file, amp);
-      fprintf(internal_structure_file, "\t%d", 1);
-      fprintf(internal_structure_file, "\t%d\t%d", 
-              ampinfo[seed].swarmid + 1,
-              ampinfo[amp].generation);
-      fprintf(internal_structure_file, "\n");
-    }
-}
-
 void add_amp_to_swarm(int amp)
 {
   /* add to swarm */
   ampinfo[current_swarm_tail].next = amp;
   current_swarm_tail = amp;
-
-  swarm_breaker_info(amp);
 }
 
-void process_seed(int seed, int subseed)
+void process_seed(int subseed)
 {
   unsigned long seqlen = db_getsequencelen(subseed);
 
-  threads_used = threads;
+  threads_used = opt_threads;
   if (threads_used > seqlen + 1)
     threads_used = seqlen+1;
 
@@ -510,7 +491,8 @@ void process_seed(int seed, int subseed)
     {
       if (global_hits_count + ti[t].hits_count > global_hits_alloc)
         {
-          global_hits_alloc <<= 1;
+          while (global_hits_count + ti[t].hits_count > global_hits_alloc)
+            global_hits_alloc <<= 1;
           global_hits_data = (int*)xrealloc(global_hits_data,
                                             global_hits_alloc * sizeof(int));
         }
@@ -648,7 +630,12 @@ int attach_candidates(int amplicons)
       int parent = graft_array[i].parent;
       int child  = graft_array[i].child;
 
-      if (!swarminfo[ampinfo[child].swarmid].attached)
+      if (swarminfo[ampinfo[child].swarmid].attached)
+        {
+          /* this light swarm is already attached */
+          ampinfo[child].graft_cand = NO_SWARM;
+        }
+      else
         {
           /* attach child to parent */
           attach(parent, child);
@@ -949,6 +936,8 @@ BloomFilter * bloomp;
 
 void mark_light_thread(long t)
 {
+  (void) t;
+
   char * buffer1 = (char*) xmalloc(db_getlongestsequence() + 2);
   pthread_mutex_lock(&light_mutex);
   while (light_progress < light_amplicon_count)
@@ -976,6 +965,8 @@ static long amplicons;
 
 void check_heavy_thread(long t)
 {
+  (void) t;
+
   char * buffer1 = (char*) xmalloc(db_getlongestsequence() + 2);
   char * buffer2 = (char*) xmalloc(db_getlongestsequence() + 3);
   pthread_mutex_lock(&heavy_mutex);
@@ -1071,7 +1062,7 @@ void algo_d1_run()
           global_hits_count = 0;
 
           /* find the first generation matches */
-          process_seed(seed, seed);
+          process_seed(seed);
 
           /* sort hits */
           qsort(global_hits_data, global_hits_count,
@@ -1089,7 +1080,7 @@ void algo_d1_run()
               global_hits_count = 0;
               while(subseed != NO_SWARM)
                 {
-                  process_seed(seed, subseed);
+                  process_seed(subseed);
                   update_stats(subseed);
                   subseed = ampinfo[subseed].next;
                 }
@@ -1227,7 +1218,7 @@ void algo_d1_run()
             }
 
           fprintf(logfile,
-                  "Bloom filter: bits=%ld, m=%ld, k=%ld, size=%.1lfMB\n",
+                  "Bloom filter: bits=%ld, m=%ld, k=%ld, size=%.1fMB\n",
                   bits, m, k, 1.0 * m / (8*1024*1024));
           
           bloomp = new BloomFilter(m, k);
@@ -1247,7 +1238,7 @@ void algo_d1_run()
           light_progress = 0;
           light_amplicon_count = amplicons_in_small_otus;
           light_amplicon = amplicons - 1;
-          ThreadRunner * tr = new ThreadRunner(threads, mark_light_thread);
+          ThreadRunner * tr = new ThreadRunner(opt_threads, mark_light_thread);
           tr->run();
           delete tr;
           pthread_mutex_destroy(&light_mutex);
@@ -1286,7 +1277,7 @@ void algo_d1_run()
           heavy_progress = 0;
           heavy_amplicon_count = amplicons_in_large_otus;
           heavy_amplicon = 0;
-          ThreadRunner * heavy_tr = new ThreadRunner(threads, 
+          ThreadRunner * heavy_tr = new ThreadRunner(opt_threads, 
                                                      check_heavy_thread);
           heavy_tr->run();
           delete heavy_tr;
@@ -1331,8 +1322,8 @@ void algo_d1_run()
 
   progress_init("Writing swarms:   ", swarmcount);
 
-  if (mothur)
-    fprintf(outfile, "swarm_%ld\t%ld", resolution, swarmcount_adjusted);
+  if (opt_mothur)
+    fprintf(outfile, "swarm_%ld\t%ld", opt_differences, swarmcount_adjusted);
 
   for(int i = 0; i < swarmcount; i++)
     {
@@ -1343,7 +1334,7 @@ void algo_d1_run()
                a >= 0;
                a = ampinfo[a].next)
             {
-              if (mothur)
+              if (opt_mothur)
                 {
                   if (a == seed)
                     fputc('\t', outfile);
@@ -1357,13 +1348,13 @@ void algo_d1_run()
                 }
               fprint_id(outfile, a);
             }
-          if (!mothur)
+          if (!opt_mothur)
             fputc('\n', outfile);
         }
       progress_update(i+1);
     }
 
-  if (mothur)
+  if (opt_mothur)
     fputc('\n', outfile);
   
   progress_done();
@@ -1373,7 +1364,7 @@ void algo_d1_run()
 
   if (opt_seeds)
     {
-      progress_init("Writing seeds:    ", swarmcount_adjusted);
+      progress_init("Writing seeds:    ", swarmcount);
       for(int i=0; i < swarmcount; i++)
         {
           if (!swarminfo[i].attached)
@@ -1390,10 +1381,80 @@ void algo_d1_run()
     }
 
 
+  /* output internal structure */
+
+  if (opt_internal_structure)
+    {
+      unsigned int cluster_no = 0;
+
+      progress_init("Writing structure:", swarmcount);
+
+      for(unsigned int swarmid = 0; swarmid < swarmcount ; swarmid++)
+        {
+          if (!swarminfo[swarmid].attached)
+            {
+              int seed = swarminfo[swarmid].seed;
+
+              struct ampinfo_s * bp = ampinfo + seed;
+
+              for (int a = bp->next;
+                   a >= 0;
+                   a = ampinfo[a].next)
+                {
+                  long graft_parent = ampinfo[a].graft_cand;
+                  if (graft_parent != NO_SWARM)
+                    {
+                      fprint_id_noabundance(internal_structure_file, graft_parent);
+                      fprintf(internal_structure_file, "\t");
+                      fprint_id_noabundance(internal_structure_file, a);
+                      fprintf(internal_structure_file,
+                              "\t%d\t%d\t%d\n",
+                              2,
+                              cluster_no + 1,
+                              ampinfo[graft_parent].generation + 1);
+                    }
+
+                  long parent = ampinfo[a].parent;
+                  if (parent != NO_SWARM)
+                    {
+                      int diff = 1;
+                      if (duplicates_found)
+                        {
+                          unsigned long parentseqlen = db_getsequencelen(parent);
+                          unsigned long ampseqlen = db_getsequencelen(a);
+                          if (parentseqlen == ampseqlen)
+                            {
+                              unsigned char * parentseq = (unsigned char *) db_getsequence(parent);
+                              unsigned char * ampseq = (unsigned char *) db_getsequence(a);
+                              if (memcmp(parentseq, ampseq, parentseqlen) == 0)
+                                diff = 0;
+                            }
+                        }
+                      fprint_id_noabundance(internal_structure_file, parent);
+                      fprintf(internal_structure_file, "\t");
+                      fprint_id_noabundance(internal_structure_file, a);
+                      fprintf(internal_structure_file,
+                              "\t%d\t%d\t%d\n",
+                              diff,
+                              cluster_no + 1,
+                              ampinfo[a].generation);
+                    }
+                }
+
+              cluster_no++;
+            }
+          progress_update(swarmid);
+        }
+      progress_done();
+    }
+
+
   /* output swarm in uclust format */
 
   if (uclustfile)
     {
+      unsigned int cluster_no = 0;
+
       progress_init("Writing UCLUST:   ", swarmcount);
 
       for(unsigned int swarmid = 0; swarmid < swarmcount ; swarmid++)
@@ -1405,13 +1466,13 @@ void algo_d1_run()
               struct ampinfo_s * bp = ampinfo + seed;
               
               fprintf(uclustfile, "C\t%u\t%d\t*\t*\t*\t*\t*\t",
-                      swarmid,
+                      cluster_no,
                       swarminfo[swarmid].size);
               fprint_id(uclustfile, seed);
               fprintf(uclustfile, "\t*\n");
               
               fprintf(uclustfile, "S\t%u\t%lu\t*\t*\t*\t*\t*\t",
-                      swarmid,
+                      cluster_no,
                       db_getsequencelen(seed));
               fprint_id(uclustfile, seed);
               fprintf(uclustfile, "\t*\n");
@@ -1440,7 +1501,7 @@ void algo_d1_run()
                   
                   fprintf(uclustfile,
                           "H\t%d\t%lu\t%.1f\t+\t0\t0\t%s\t",
-                          ampinfo[seed].swarmid,
+                          cluster_no,
                           db_getsequencelen(a),
                           percentid, 
                           nwdiff > 0 ? nwalignment : "=");
@@ -1453,6 +1514,8 @@ void algo_d1_run()
                   if (nwalignment)
                     free(nwalignment);
                 }
+
+              cluster_no++;
             }
           progress_update(swarmid);
         }
