@@ -90,21 +90,31 @@ void showseq(char * seq)
 
 void fprint_id(FILE * stream, unsigned long x)
 {
-  fprintf(stream, "%.*s", seqindex[x].headeridlen, seqindex[x].header);
+  seqinfo_t * sp = seqindex + x;
+  char * h = sp->header;
+  int hdrlen = sp->headerlen;
+
+  if (opt_append_abundance && (sp->abundance_start == sp->abundance_end))
+    if (opt_usearch_abundance)
+      fprintf(stream, "%.*s;size=%lu;", hdrlen, h, sp->abundance);
+    else
+      fprintf(stream, "%.*s_%lu", hdrlen, h, sp->abundance);
+  else
+    fprintf(stream, "%.*s", hdrlen, h);
 }
 
 void fprint_id_noabundance(FILE * stream, unsigned long x)
 {
   seqinfo_t * sp = seqindex + x;
   char * h = sp->header;
-  int hdrlen = sp->headeridlen;
+  int hdrlen = sp->headerlen;
 
   if (sp->abundance_start < sp->abundance_end)
     {
       /* print start of header */
       fprintf(stream, "%.*s", sp->abundance_start, h);
       
-      if (usearch_abundance)
+      if (opt_usearch_abundance)
         {
           /* print semicolon if the abundance is not at either end */
           if ((sp->abundance_start > 0) && (sp->abundance_end < hdrlen))
@@ -115,9 +125,7 @@ void fprint_id_noabundance(FILE * stream, unsigned long x)
         }
     }
   else
-    {
-      fprintf(stream, "%s", h);
-    }
+    fprintf(stream, "%.*s", hdrlen, h);
 }
 
 void fprint_id_with_new_abundance(FILE * stream,
@@ -126,14 +134,14 @@ void fprint_id_with_new_abundance(FILE * stream,
 {
   seqinfo_t * sp = seqindex + seqno;
 
-  if (usearch_abundance)
+  if (opt_usearch_abundance)
     fprintf(stream,
             "%.*s%ssize=%lu;%.*s",
             sp->abundance_start,
             sp->header,
             sp->abundance_start > 0 ? ";" : "",
             abundance,
-            sp->headeridlen - sp->abundance_end,
+            sp->headerlen - sp->abundance_end,
             sp->header + sp->abundance_end);
   else
     fprintf(stream,
@@ -153,14 +161,7 @@ int db_compare_abundance(const void * a, const void * b)
   else if (x->abundance < y->abundance)
     return +1;
   else 
-    {
-      if (x < y)
-        return -1;
-      else if (x > y)
-        return +1;
-      else
-        return 0;
-    }
+    return strcmp(x->header, y->header);
 }
 
 void db_read(const char * filename)
@@ -189,14 +190,15 @@ void db_read(const char * filename)
 
   /* get file size */
 
-  long filesize = 0;
-  if (filename)
-    {
-      if (fseek(fp, 0, SEEK_END))
-        fatal("Error: Unable to seek in database file (%s)", filename);
-      filesize = ftell(fp);
-      rewind(fp);
-    }
+  struct stat fs;
+
+  if (fstat(fileno(fp), & fs))
+    fatal("Unable to fstat on input file (%s)", filename);
+  bool is_regular = S_ISREG(fs.st_mode);
+  long filesize = is_regular ? fs.st_size : 0;
+
+  if (! is_regular)
+    fprintf(logfile, "Waiting for data... (Hit Ctrl-C and run swarm -h if you meant to read data from a file.)\n");
 
   char line[LINEALLOC];
   line[0] = 0;
@@ -263,10 +265,11 @@ void db_read(const char * filename)
 
       while (line[0] && (line[0] != '>'))
         {
-          char m;
           char c;
           char * p = line;
           while((c = *p++))
+	    {
+	    char m;
             if ((m = map_nt[(int)c]) >= 0)
               {
                 while (datalen >= dataalloc)
@@ -280,13 +283,19 @@ void db_read(const char * filename)
               }
             else if ((c != 10) && (c != 13))
               {
-                char msg[100];
                 if ((c >= 32) && (c <= 126))
-                  snprintf(msg, 100, "Illegal character '%c' in sequence on line %u", c, lineno);
+                  fprintf(stderr,
+                          "Illegal character '%c' in sequence on line %u",
+                          c,
+                          lineno);
                 else
-                  snprintf(msg, 100, "Illegal character (ascii no %d) in sequence on line %u", c, lineno);
-                fatal(msg);
+                  fprintf(stderr,
+                          "Illegal character (ascii no %d) in sequence on line %u",
+                          c,
+                          lineno);
+                exit(1);
               }
+	    }
           line[0] = 0;
           if (!fgets(line, LINEALLOC, fp))
             line[0] = 0;
@@ -301,6 +310,12 @@ void db_read(const char * filename)
       
       long length = datalen - seqbegin;
 
+      if (length == 0)
+        {
+          fprintf(stderr, "\nError: Empty sequence found on line %u.\n\n", lineno-1);
+          exit(1);
+        }
+
       nucleotides += length;
 
       if (length > longest)
@@ -311,7 +326,7 @@ void db_read(const char * filename)
 
       sequences++;
       
-      if (filename)
+      if (is_regular)
         progress_update(ftell(fp));
     }
   progress_done();
@@ -322,12 +337,25 @@ void db_read(const char * filename)
 
   unsigned long hdrhashsize = 2 * sequences;
 
-  seqinfo_t * * hdrhashtable = 
+  seqinfo_t * * hdrhashtable =
     (seqinfo_t **) xmalloc(hdrhashsize * sizeof(seqinfo_t *));
   memset(hdrhashtable, 0, hdrhashsize * sizeof(seqinfo_t *));
 
   unsigned long duplicatedidentifiers = 0;
-  
+
+  /* set up hash to check for unique sequences */
+
+  unsigned long seqhashsize = 2 * sequences;
+
+  seqinfo_t * * seqhashtable = 0;
+
+  if (opt_differences > 0)
+    {
+      seqhashtable =
+        (seqinfo_t **) xmalloc(seqhashsize * sizeof(seqinfo_t *));
+      memset(seqhashtable, 0, seqhashsize * sizeof(seqinfo_t *));
+    }
+
   /* create indices */
 
   seqindex = (seqinfo_t *) xmalloc(sequences * sizeof(seqinfo_t));
@@ -336,7 +364,7 @@ void db_read(const char * filename)
   regex_t db_regexp;
   regmatch_t pmatch[4];
 
-  if (usearch_abundance)
+  if (opt_usearch_abundance)
     {
       if (regcomp(&db_regexp, "(^|;)size=([0-9]+)(;|$)", REG_EXTENDED))
         fatal("Regular expression compilation failed");
@@ -347,11 +375,12 @@ void db_read(const char * filename)
         fatal("Regular expression compilation failed");
     }
 
-  long lastabundance = LONG_MAX;
+  seqinfo_t * lastseq = 0;
 
   int presorted = 1;
   int missingabundance = 0;
   unsigned int missingabundance_lineno = 0;
+  char * missingabundance_header = 0;
 
   char * p = datap;
   progress_init("Indexing database:", sequences);
@@ -364,26 +393,40 @@ void db_read(const char * filename)
       /* get header */
       seqindex_p->header = p;
       seqindex_p->headerlen = strlen(seqindex_p->header);
-      seqindex_p->headeridlen = seqindex_p->headerlen;
       p += seqindex_p->headerlen + 1;
 
+      /* and sequence */
+      seqindex_p->seq = p;
+      seqindex_p->seqlen = strlen(p);
+      p += seqindex_p->seqlen + 1;
+
       /* get amplicon abundance */
-      seqindex_p->abundance = 0;
       if (!regexec(&db_regexp, seqindex_p->header, 4, pmatch, 0))
         {
           seqindex_p->abundance = atol(seqindex_p->header + pmatch[2].rm_so);
           seqindex_p->abundance_start = pmatch[0].rm_so;
           seqindex_p->abundance_end = pmatch[0].rm_eo;
+
+          if (seqindex_p->abundance == 0)
+            {
+              fprintf(stderr,
+                      "\nError: Illegal abundance value on line %u:\n%s\n"
+                      "Abundance values should be positive integers.\n\n",
+                      lineno,
+                      seqindex_p->header);
+              exit(1);
+            }
         }
       else
         {
-          seqindex_p->abundance_start = 0;
-          seqindex_p->abundance_end = 0;
+          seqindex_p->abundance_start = seqindex_p->headerlen;
+          seqindex_p->abundance_end = seqindex_p->headerlen;
+          seqindex_p->abundance = 0;
         }
       
       if (seqindex_p->abundance < 1)
         {
-          if (opt_append_abundance > 0)
+          if (opt_append_abundance)
             {
               seqindex_p->abundance = opt_append_abundance;
             }
@@ -391,42 +434,85 @@ void db_read(const char * filename)
             {
               missingabundance++;
               if (missingabundance == 1)
-                missingabundance_lineno = lineno;
+                {
+                  missingabundance_lineno = lineno;
+                  missingabundance_header = seqindex_p->header;
+                }
             }
         }
 
-      if (seqindex_p->abundance > lastabundance)
-        presorted = 0;
+      if (seqindex_p->abundance_start == 0)
+          fatal("Empty sequence identifier");
 
-      lastabundance = seqindex_p->abundance;
 
-      /* check hash, fatal error if found, otherwize insert new */
-      unsigned long hdrhash = HASH((unsigned char*)seqindex_p->header, seqindex_p->headeridlen);
-      seqindex_p->hdrhash = hdrhash;
-      unsigned long hashindex = hdrhash % hdrhashsize;
+      /* check if the sequences are presorted by abundance and header */
 
-      seqinfo_t * found;
-    
-      while ((found = hdrhashtable[hashindex]))
+      if (presorted && lastseq)
         {
-          if ((found->hdrhash == hdrhash) &&
-              (found->headeridlen == seqindex_p->headeridlen) &&
-              (strncmp(found->header, seqindex_p->header, found->headeridlen) == 0))
-            break;
-          hashindex = (hashindex + 1) % hdrhashsize;
+          if (lastseq->abundance < seqindex_p->abundance)
+            presorted = 0;
+          else if (lastseq->abundance == seqindex_p->abundance)
+            {
+              if (strcmp(lastseq->header, seqindex_p->header) > 0)
+                presorted = 0;
+            }
         }
 
-      if (found)
+      lastseq = seqindex_p;
+
+
+      /* check for duplicated identifiers using hash table */
+
+      unsigned long hdrhash = HASH((unsigned char*)seqindex_p->header, seqindex_p->abundance_start);
+      seqindex_p->hdrhash = hdrhash;
+      unsigned long hdrhashindex = hdrhash % hdrhashsize;
+
+      seqinfo_t * hdrfound = 0;
+    
+      while ((hdrfound = hdrhashtable[hdrhashindex]))
+        {
+          if ((hdrfound->hdrhash == hdrhash) &&
+              (hdrfound->abundance_start == seqindex_p->abundance_start) &&
+              (strncmp(hdrfound->header, seqindex_p->header, hdrfound->abundance_start) == 0))
+            break;
+          hdrhashindex = (hdrhashindex + 1) % hdrhashsize;
+        }
+
+      if (hdrfound)
         {
           duplicatedidentifiers++;
-          fprintf(logfile, "WARNING: Duplicated sequence identifier: %s\n", seqindex_p->header);
+          fprintf(stderr, "\nError: Duplicated sequence identifier: %.*s\n\n",
+                  seqindex_p->abundance_start,
+                  seqindex_p->header);
+          exit(1);
         }
 
-      hdrhashtable[hashindex] = seqindex_p;
+      hdrhashtable[hdrhashindex] = seqindex_p;
     
-      seqindex_p->seq = p;
-      seqindex_p->seqlen = strlen(p);
-      p += seqindex_p->seqlen + 1;
+
+      if (opt_differences > 0)
+        {
+          /* check for duplicated sequences using hash table */
+          unsigned long seqhash = HASH((unsigned char*)seqindex_p->seq,
+                                       seqindex_p->seqlen);
+          seqindex_p->seqhash = seqhash;
+          unsigned long seqhashindex = seqhash % seqhashsize;
+          seqinfo_t * seqfound = 0;
+
+          while ((seqfound = seqhashtable[seqhashindex]))
+            {
+              if ((seqfound->seqhash == seqhash) &&
+                  (seqfound->seqlen == seqindex_p->seqlen) &&
+                  (memcmp(seqfound->seq, seqindex_p->seq, seqfound->seqlen) == 0))
+                break;
+              seqhashindex = (seqhashindex + 1) % seqhashsize;
+            }
+
+          if (seqfound)
+            duplicates_found++;
+          else
+            seqhashtable[seqhashindex] = seqindex_p;
+        }
 
       seqindex_p++;
       progress_update(i);
@@ -435,17 +521,26 @@ void db_read(const char * filename)
 
   if (missingabundance)
     {
-      char * msg;
-      if (asprintf(&msg,
-                   "Abundance annotations not found for %d sequences, starting on line %u.\n"
-                   "Fasta headers must end with abundance annotations (_INT or ;size=INT).\n"
-                   "The -z option must be used if the abundance annotation is in the latter format.\n"
-                   "Abundance annotations can be produced by dereplicating the sequences.\n",
-                   missingabundance,
-                   missingabundance_lineno) == -1)
-        fatal("Out of memory");
-      else
-        fatal(msg);
+      fprintf(stderr,
+              "\nError: Abundance annotations not found for %d sequences, starting on line %u.\n"
+              ">%s\n"
+              "Fasta headers must end with abundance annotations (_INT or ;size=INT).\n"
+              "The -z option must be used if the abundance annotation is in the latter format.\n"
+              "Abundance annotations can be produced by dereplicating the sequences.\n"
+              "The header is defined as the string comprised between the \">\" symbol\n"
+              "and the first space or the end of the line, whichever comes first.\n\n",
+              missingabundance,
+              missingabundance_lineno,
+              missingabundance_header);
+      exit(1);
+    }
+
+  if (duplicates_found)
+    {
+      fprintf(logfile,
+              "WARNING: %lu duplicated sequences detected.\n"
+              "Please consider dereplicating your data for optimal results.\n",
+              duplicates_found);
     }
 
   if (!presorted)
@@ -459,8 +554,11 @@ void db_read(const char * filename)
 
   free(hdrhashtable);
 
-  if (duplicatedidentifiers)
-    exit(1);
+  if (seqhashtable)
+    {
+      free(seqhashtable);
+      seqhashtable = 0;
+    }
 }
 
 void db_qgrams_init()
